__version__ = '4.2'
VERSION = tuple(map(int, __version__.split('.')))
# NOTE read here first https://github.com/Suor/django-cacheops/issues/35


import time
from django.apps import AppConfig
from cacheops.simple import *
from cacheops.query import *
from cacheops.invalidation import *
from cacheops.templatetags.cacheops import *
from funcy import monkey, decorator
from cacheops.transaction import install_cacheops_transaction_support
from rediscluster.exceptions import ClusterError, RedisClusterException, ClusterDownError
import redis

import logging
logger = logging.getLogger("api.activity")

_last_ping = None

# NOTE read here first https://github.com/Suor/django-cacheops/issues/35

import cacheops.redis
@monkey(cacheops.redis, 'handle_connection_failure')
@decorator
def handle_connection_failure(call):
    global _last_ping
    if _last_ping and time.time() - _last_ping < 60:
        # redis connection down, we dont  want to keep trying, just go directly to db, up until 60s, try again
        return
    try:
        _last_ping = None
        return call()
    except (ClusterError, RedisClusterException, ClusterDownError, redis.ConnectionError, redis.TimeoutError) as err:
        _last_ping = time.time()
        logger.exception(f'#### Redis cluster not available {str(err)}')


class CacheopsClusterConfig(AppConfig):
    name = 'compat_cacheops'

    def ready(self):
        # here I do monkey patch those methods, we need to somehow load it before the install_cacheops() called
        from compat_cacheops.query import cache_thing
        from compat_cacheops.invalidation import redis_can_unlink
        install_cacheops()
        install_cacheops_transaction_support()


default_app_config = 'compat_cacheops.CacheopsClusterConfig'
