import json
from cacheops.transaction import transaction_states
from cacheops.redis import handle_connection_failure
from cacheops.conf import settings
from cacheops.cross import pickle
from cacheops.redis import redis_client
import re
import os.path
from funcy import memoize, monkey
import cacheops


STRIP_RE = re.compile(r'TOSTRIP.*/TOSTRIP', re.S)


# NOTE read here first https://github.com/Suor/django-cacheops/issues/35

@monkey(cacheops.query, name='cache_thing')
@handle_connection_failure
def cache_thing(prefix, cache_key, data, cond_dnfs, timeout, dbs=(), precall_key=''):
    """
    Writes data to cache and creates appropriate invalidators.
    If precall_key is not the empty string, the data will only be cached if the
    precall_key is set to avoid caching stale data.
    """
    if transaction_states.is_dirty(dbs):
        return
    load_script('cache_thing', settings.CACHEOPS_LRU)(
        keys=[prefix, cache_key],
        args=[
            precall_key,
            pickle.dumps(data, -1),
            json.dumps(cond_dnfs, default=str),
            timeout
        ]
    )

@memoize
def load_script(name, strip=False):
    filename = os.path.join(os.path.dirname(__file__), 'lua/%s.lua' % name)
    with open(filename) as f:
        code = f.read()
    if strip:
        code = STRIP_RE.sub('', code)
    return redis_client.register_script(code)
